// Rover R7 v1.0
// Build instruction on:
// http://homofaciens.de/technics-robots-R7-construction_en.htm

base_mount_l = 63;
base_mount_w1 = 59;
base_mount_w2 = 66;
base_mount_edge = 45;
base_mount_edge_2 = 5;
base_mount_d = 5;
mount_front_l = 49;
mount_front_h = 10;
mount_front_w = 15;
base_mount_screw = 2;
base_w = 85;
base_l = 135;
base_h = 32;
wall_thickness = 3;
$fn=100;
raspberry_mount_l = 58;
raspberry_mount_w = 49;
raspberry_mount_edge = 60;
raspberry_mount_d = 5;
raspberry_mount_screw_d = 2;
raspberry_mount_d_tolerance = 0.3;
raspberry_mount_h = 9;
raspberry_mount_socket = 1;

cable_hole_d = 15;
cable_hole_l = 80;
cable_hole_w = base_w/2 - cable_hole_d/2 - 5;

camera_w = 26.5;
camera_h = 25.5;
camera_thickness = 10;
camera_board_thickness = 1.5;
camera_wall = 2;
camera_light_mount_w = 6;
camera_light_mount_hole = 3.2;

light_mount_w = 20;
light_d = 5.3;
light_edge = light_d/2+2;

body_mount_w = 75;
body_mount_l = 125;

/*
base();

translate([-30, 55, 0]){
  camera_mount();
}

translate([-30, 75, 0]){
  light_mount();
}
translate([30, 75, 0]){
  light_mount();
}
*/
translate([0, -90, 0]){
  rotate([0, 0, 90]){
    body();
  }
}

/*
//test wall thickness
cube(size=[20, 0.7, 20]);
translate([0, 10, 0]){
  cube(size=[20, 0.6, 20]);
}
translate([0, 0, 0]){
  cube(size=[20, 10, 0.5]);
}
*/

//battery_holder();

module battery_holder(){
  bat_no = 4;
  bat_l = 50.0;
  bat_d = 14.5;
  plus_terminal_d = 7;
  minus_terminal_h = 1;
  minus_terminal_d = 8;
  bat_wall = 0.8;
  bat_wall_base = 2;
  bat_wall_2 = 2;
  bat_cut_w = 2.0;
  bat_cut_l = bat_l*0.4;
  bat_slit=1.5;
  
  cable_hole=2;

  
  wall_height = bat_d-2;
  
  width=bat_no*(bat_d+bat_wall)+bat_wall;
  length=bat_l+bat_wall+bat_wall_2;
  
  wall_l = length-bat_cut_l-bat_wall/2;
  
  translate([0, 0, bat_wall_base*0]){
    difference(){
      translate([0, 0, bat_wall_base/2]){
        cube(size=[length, width, bat_wall_base], center=true);
      }
      for(i=[0:1:bat_no]){
        translate([(length-bat_cut_l)/2+0.1, width/2-i*(bat_d+bat_wall)-bat_wall/2, 0]){
          cube(size=[bat_cut_l+0.2, bat_cut_w+0.1, bat_wall_base*2.5], center=true);
        }
      }
      for(i=[1:1:bat_no]){
        translate([-(bat_wall_2-bat_wall)/2, width/2-i*(bat_d+bat_wall)-(-bat_d)/2, bat_d/2]){
          rotate([0, 90, 0]){
            cylinder(d=bat_d+0.5, h=bat_l, center=true);
          }
        }
      }
    }
    
    for(i=[0:1:bat_no]){
      //walls
      translate([(bat_wall-length+wall_l)/2, width/2-i*(bat_d+bat_wall)-bat_wall/2, wall_height/2]){
        cube(size=[wall_l, bat_wall, wall_height], center=true);
      }
    }
    
    for(i=[1:1:bat_no]){
      //positive terminals
      translate([(length-bat_wall_2)/2, width/2-i*(bat_d+bat_wall)-(-bat_d)/2, bat_d/2]){
        difference(){
          union(){
            rotate([0, 90, 0]){
              cylinder(d=bat_d-(bat_cut_w-bat_wall), h=bat_wall_2, center=true);
            }
            translate([0, 0, -bat_d/4]){
              cube(size=[bat_wall_2, bat_d-(bat_cut_w-bat_wall), bat_d/2], center=true);
            }
          }
          translate([-0.8, 0, 0]){
            //rotate([0, 90, 0]){
              cube(size=[bat_wall_2, plus_terminal_d, plus_terminal_d], center=true);
            //}
          }
          translate([0, 0, (plus_terminal_d-bat_slit)/2]){
            cube(size=[bat_wall_2*2, plus_terminal_d, bat_slit], center=true);
          }
          translate([0, 0, -(plus_terminal_d-bat_slit)/2]){
            cube(size=[bat_wall_2*2, plus_terminal_d, bat_slit], center=true);
          }
          translate([0, 0, bat_d/2]){
            cube(size=[bat_wall_2*2, width, 5], center=true);
          }
        }
      }
    }
    translate([(bat_wall-length)/2, 0, 0]){
      difference(){
        translate([0, 0, wall_height/2]){
          cube(size=[bat_wall, width, wall_height], center=true);
        }
        //negative terminals
        for(i=[1:1:bat_no]){
          translate([0, width/2-i*(bat_d+bat_wall)-(bat_wall-bat_d)/2, (bat_d+plus_terminal_d-bat_slit)/2]){
            cube(size=[bat_wall*2, plus_terminal_d, bat_slit], center=true);
          }
          translate([0, width/2-i*(bat_d+bat_wall)-(bat_wall-bat_d)/2, (bat_d-plus_terminal_d+bat_slit)/2]){
            cube(size=[bat_wall*2, plus_terminal_d, bat_slit], center=true);
          }
          translate([0, width/2-(i-1)*(bat_d+bat_wall)-(cable_hole)/2-bat_wall, cable_hole/2+bat_wall_base]){
            rotate([0, 90, 0]){
              cylinder(d=cable_hole, h=bat_wall*2, center=true);
            }
          }
        }
        
      }
    }
  }
  
}

module body(){
  body_width = 90;
  body_length = 197;
  body_height = 60;
  body_wall = 0.8;
  body_wall_f = 5;
  
  body_mount_d = 6;
  body_mount_hole = 3;
  
  front_wheel_cut_h = 15;
  front_wheel_cut_window_t = 15;
  front_wheel_cut_window_b = 30;
  front_wheel_cut_window = 10;
  wheel_space = 90;
  wheel_space_height = 20 + front_wheel_cut_h;
  wheel_space_edge = 60;
  
  top_height = 5;
  cutting_edge = 5;
  cutting_edge_i = 5;
  scale_x = body_width/(body_width-top_height*2);
  scale_y = body_length/(body_length-top_height*2);
  
  mount_height = 20;
  mount_edge = 55;
  
  
  x1=body_width/2;
  y1=body_length/2-cutting_edge;
  x2=body_width/2-cutting_edge;
  y2=body_length/2;
  x3=-body_width/2+cutting_edge;
  y3=body_length/2;
  x4=-body_width/2;
  y4=body_length/2-cutting_edge;
  x5=-body_width/2;
  y5=-body_length/2+cutting_edge;
  x6=-body_width/2+cutting_edge;
  y6=-body_length/2;
  x7=body_width/2-cutting_edge;
  y7=-body_length/2;
  x8=body_width/2;
  y8=-body_length/2+cutting_edge;
  
  
  translate([0, 0, 0]){
    scale(v=[1/scale_x, 1/scale_y, 1]){
      linear_extrude(height=body_wall, scale=[1, 1]){
         polygon(points=[[x1,y1],[x2,y2],[x3,y3],[x4,y4],[x5,y5],[x6,y6],[x7,y7],[x8,y8]]);
      }
    }
  }


  translate([0, 0, top_height]){
    mirror([0, 0, 1]){
      linear_extrude(height=top_height, scale=[1/scale_x, 1/scale_y]){
        difference(){
            polygon(points=[[x1,y1],[x2,y2],[x3,y3],[x4,y4],[x5,y5],[x6,y6],[x7,y7],[x8,y8]]);
        polygon(points=[[x1-body_wall,y1-cutting_edge_i],
                         [x2,y2-body_wall_f],
                         [x3,y3-body_wall_f],
                         [x4+body_wall,y4-cutting_edge_i],
                         [x5+body_wall,y5+cutting_edge_i],
                         [x6+cutting_edge_i,y6+body_wall],
                         [x7-cutting_edge_i,y7+body_wall],
                         [x8-body_wall,y8+cutting_edge_i]]);
          
            //polygon(points=[[x1-body_wall,y1],[x2,y2-body_wall_f],[x3,y3-body_wall_f],[x4+body_wall,y4],[x5+body_wall,y5],[x6,y6+body_wall],[x7,y7+body_wall],[x8-body_wall,y8]]);
        }
      }
    }
  }

  translate([0, (body_length-body_wall_f)/2, (body_height+top_height)/2]){
    window_frame(body_width-cutting_edge*2, body_height-top_height, body_wall_f);
  }

  translate([0, 0, top_height]){
    linear_extrude(height=body_height-top_height-front_wheel_cut_h, scale=[1, 1]){
      difference(){
        polygon(points=[[x1,y1],[x2,y2],[x3,y3],[x4,y4],[x5,y5],[x6,y6],[x7,y7],[x8,y8]]);
        polygon(points=[[x1-body_wall,y1-cutting_edge_i],
                         [x2,y2-body_wall_f],
                         [x3,y3-body_wall_f],
                         [x4+body_wall,y4-cutting_edge_i],
                         [x5+body_wall,y5+cutting_edge_i],
                         [x6+cutting_edge_i,y6+body_wall],
                         [x7-cutting_edge_i,y7+body_wall],
                         [x8-body_wall,y8+cutting_edge_i]]);
        translate([0, body_length/2, 0]){
          square(size=[body_width-cutting_edge*2, body_wall_f*4], center=true);
        }
      }
    }
  }

  //rear half
  translate([0, 0, top_height]){
    linear_extrude(height=body_height-top_height, scale=[1, 1]){
      difference(){
        polygon(points=[[x4,0],[x5,y5],[x6,y6],[x7,y7],[x8,y8],[-x4,0]]);
        polygon(points=[[x1-body_wall,y1-cutting_edge_i],
                         [x2,y2-body_wall_f],
                         [x3,y3-body_wall_f],
                         [x4+body_wall,y4-cutting_edge_i],
                         [x5+body_wall,y5+cutting_edge_i],
                         [x6+cutting_edge_i,y6+body_wall],
                         [x7-cutting_edge_i,y7+body_wall],
                         [x8-body_wall,y8+cutting_edge_i]]);
        translate([0, body_length/2, 0]){
          square(size=[body_width-cutting_edge*2, body_wall_f*4], center=true);
        }
      }
    }
  }

  //front half
  translate([0, 0, top_height]){
    linear_extrude(height=body_height-top_height, scale=[1, 1]){
      difference(){
        polygon(points=[[-x5,body_length/2-front_wheel_cut_window],[x1,y1],[x2,y2],[x3,y3],[x4,y4],[x5,body_length/2-front_wheel_cut_window]]);
        polygon(points=[[x4,0],[x5,y5],[x6,y6],[x7,y7],[x8,y8],[-x4,0]]);
        polygon(points=[[x1-body_wall,y1-cutting_edge_i],
                         [x2,y2-body_wall_f],
                         [x3,y3-body_wall_f],
                         [x4+body_wall,y4-cutting_edge_i],
                         [x5+body_wall,y5+cutting_edge_i],
                         [x6+cutting_edge_i,y6+body_wall],
                         [x7-cutting_edge_i,y7+body_wall],
                         [x8-body_wall,y8+cutting_edge_i]]);
        translate([0, body_length/2, 0]){
          square(size=[body_width-cutting_edge*2, body_wall_f*4], center=true);
        }
      }
    }
  }

  translate([(body_width-body_wall)/2, (body_length-front_wheel_cut_window_t-cutting_edge)/2, body_height-front_wheel_cut_h]){
    linear_extrude(height=front_wheel_cut_h, scale=[1, 0]){
      translate([0, -(front_wheel_cut_window_b-front_wheel_cut_window_t-cutting_edge)/2, 0]){
        square(size=[body_wall, front_wheel_cut_window_b-front_wheel_cut_window_t-cutting_edge], center=true);
      }
    }
  }
  translate([-(body_width-body_wall)/2, (body_length-front_wheel_cut_window_t-cutting_edge)/2, body_height-front_wheel_cut_h]){
    linear_extrude(height=front_wheel_cut_h, scale=[1, 0]){
      translate([0, -(front_wheel_cut_window_b-front_wheel_cut_window_t-cutting_edge)/2, 0]){
        square(size=[body_wall, front_wheel_cut_window_b-front_wheel_cut_window_t-cutting_edge], center=true);
      }
    }
  }

  

  translate([body_mount_w/2, body_length/2-mount_edge, 0]){
    body_mount(top_height, body_height-mount_height, body_mount_d, body_mount_hole);
  }
  translate([body_mount_w/2, body_length/2-mount_edge-body_mount_l, 0]){
    body_mount(top_height, body_height-mount_height, body_mount_d, body_mount_hole);
  }
  mirror([1, 0, 0]){
    translate([body_mount_w/2, body_length/2-mount_edge, 0]){
      body_mount(top_height, body_height-mount_height, body_mount_d, body_mount_hole);
    }
    translate([body_mount_w/2, body_length/2-mount_edge-body_mount_l, 0]){
      body_mount(top_height, body_height-mount_height, body_mount_d, body_mount_hole);
    }
  }
  
/*  
  translate([(body_width)/2, (body_length)/2-wheel_space*0.15-wheel_space_edge, 0]){
    body_enforcement(top_height, body_height+wheel_space_height);
  }
*/
  
  translate([(body_width-body_wall)/2, (body_length-wheel_space)/2-wheel_space_edge, body_height-front_wheel_cut_h]){
    linear_extrude(height=wheel_space_height, scale=[1, 0.8]){
      square(size=[body_wall, wheel_space], center=true);
    }
  }
  translate([-(body_width-body_wall)/2, (body_length-wheel_space)/2-wheel_space_edge, body_height-front_wheel_cut_h]){
    linear_extrude(height=wheel_space_height, scale=[1, 0.8]){
      square(size=[body_wall, wheel_space], center=true);
    }
  }
  
}

module window_frame(window_w, window_h, window_thickness){
  frame_width = 5;
  hole_d = 3;
  hole_n = 4;
  glass_thickness = 1.0;
  
  difference(){
    cube(size=[window_w, window_thickness, window_h], center=true);
    translate([0, 0, frame_width/2+0.1]){
      cube(size=[window_w-frame_width*2, window_thickness*2, window_h-frame_width], center=true);
      cube(size=[window_w-frame_width*2+4, glass_thickness, window_h-frame_width+4], center=true);
    }
    /*
    for(i=[0:1:hole_n-1]){
      translate([(window_w-frame_width)/2, 0, window_h/2-(i+0.5)*window_h/hole_n]){
        rotate([90, 0, 0]){
          cylinder(d=hole_d, h=window_thickness*2, center=true);
        }
      }
    }
    for(i=[0:1:hole_n-1]){
      translate([-(window_w-frame_width)/2, 0, window_h/2-(i+0.5)*window_h/hole_n]){
        rotate([90, 0, 0]){
          cylinder(d=hole_d, h=window_thickness*2, center=true);
        }
      }
    }
    for(i=[0:1:hole_n-1]){
      translate([window_w/2-(i+0.5)*window_w/hole_n, 0, (-window_h+frame_width)/2]){
        rotate([90, 0, 0]){
          cylinder(d=hole_d, h=window_thickness*2, center=true);
        }
      }
    }
    */
  }
  
}

module body_mount(top_height, body_mount_height, body_mount_d, body_mount_hole){
  translate([0, 0, body_mount_height/2]){
    difference(){
      cylinder(d=body_mount_d, h=body_mount_height, center=true);
      translate([0, 0, 5]){
        cylinder(d=body_mount_hole, h=body_mount_height, center=true);
      }
    }
  }
  translate([top_height/2, 0, top_height]){
    mirror([0, 0, 1]){
      linear_extrude(height=top_height, scale=[0, 1]){
        translate([top_height/2, 0, 0]){
          square(size=[top_height, 2], center=true);
        }
      }
    }
  }
  translate([top_height, 0, top_height]){
    linear_extrude(height=body_mount_height-top_height, scale=[1, 1]){
      square(size=[top_height, 2], center=true);
    }
  }
}

module body_enforcement(top_height, body_mount_height){
  translate([-top_height, 0, top_height]){
    mirror([0, 0, 1]){
      linear_extrude(height=top_height, scale=[0, 1]){
        translate([top_height/2, 0, 0]){
          square(size=[top_height, 2], center=true);
        }
      }
    }
  }
  translate([-top_height/2, 0, top_height]){
    linear_extrude(height=body_mount_height-top_height, scale=[1, 1]){
      square(size=[top_height, 2], center=true);
    }
  }
}

module light_mount(){
  hole_distance = camera_h*0.4*2;
  mount_l=camera_h + camera_wall+8;
  translate([0, 0, camera_wall/2]){
    difference(){
      cube(size=[mount_l, light_mount_w, camera_wall], center=true);
      for(i=[0:1:3]){
        translate([mount_l/2-mount_l/4*(i+1)+mount_l/8, light_mount_w/2-light_edge, 0]){
          cylinder(d=light_d, h=camera_wall*2, center=true);
        }
      }

      translate([hole_distance/2, -light_mount_w/2+3, 0]){
        cylinder(d=3.2, h=camera_wall*2, center=true);
      }
      translate([-hole_distance/2, -light_mount_w/2+3, 0]){
        cylinder(d=3.2, h=camera_wall*2, center=true);
      }

    }
  }
}

module camera_mount(){
  x0=0;
  y0=camera_thickness/2+camera_wall;
  x1=camera_w/2+camera_wall;
  y1=camera_thickness/2+camera_wall;
  x2=camera_w/2+camera_wall;
  y2=-camera_thickness/2;
  x3=camera_w/2-2;
  y3=-camera_thickness/2;
  x4=camera_w/2-2;
  y4=-camera_board_thickness/2;
  x5=camera_w/2;
  y5=-camera_board_thickness/2;
  x6=camera_w/2;
  y6=camera_board_thickness/2;
  x7=camera_w/2-2;
  y7=camera_board_thickness/2;
  x8=camera_w/2-2;
  y8=camera_thickness/2;
  x9=0;
  y9=camera_thickness/2;
  
  linear_extrude(height=camera_h){
    polygon(points=[[x0,y0],[x1,y1],[x2,y2],[x3,y3],[x4,y4],[x5,y5],[x6,y6],[x7,y7],[x8,y8],[x9,y9]]);
    polygon(points=[[-x0,y0],[-x1,y1],[-x2,y2],[-x3,y3],[-x4,y4],[-x5,y5],[-x6,y6],[-x7,y7],[-x8,y8],[-x9,y9]]);
  }
  translate([0, 0, camera_h]){
    linear_extrude(height=camera_wall){
      polygon(points=[[x0,y0],[x1,y1],[x2,y2],[x3,y3],[x8,y8],[x9,y9]]);
      polygon(points=[[-x0,y0],[-x1,y1],[-x2,y2],[-x3,y3],[-x8,y8],[-x9,y9]]);
    }
  }
  translate([0, (camera_thickness + camera_wall)/2, camera_h + (base_h-mount_front_h)/2+camera_wall]){
    difference(){
      cube(size=[mount_front_w,camera_wall, base_h-mount_front_h], center=true);
      rotate([90, 90, 0]){
        cylinder(d=3.2, h=10, center=true);
      }
    }
  }
  translate([(camera_w+camera_light_mount_w)/2+camera_wall, 0, (camera_h+camera_wall)/2]){
    difference(){
      cube(size=[camera_light_mount_w,camera_wall, camera_h+camera_wall], center=true);
      translate([0, 0, -camera_h*0.4]){
        rotate([90, 90, 0]){
          cylinder(d=3.2, h=10, center=true);
        }
      }
      translate([0, 0, camera_h*0.4]){
        rotate([90, 90, 0]){
          cylinder(d=3.2, h=10, center=true);
        }
      }
    }
  }
  translate([-(camera_w+camera_light_mount_w)/2-camera_wall, 0, (camera_h+camera_wall)/2]){
    difference(){
      cube(size=[camera_light_mount_w,camera_wall, camera_h+camera_wall], center=true);
      translate([0, 0, -camera_h*0.4]){
        rotate([90, 90, 0]){
          cylinder(d=3.2, h=10, center=true);
        }
      }
      translate([0, 0, camera_h*0.4]){
        rotate([90, 90, 0]){
          cylinder(d=3.2, h=10, center=true);
        }
      }
    }
  }
}

module base(){
  translate([base_l/2-base_mount_edge-base_mount_l-mount_front_l, 0, (base_h-mount_front_h)/2]){
    difference(){
      cube(size=[mount_front_w, mount_front_w, base_h-mount_front_h], center=true);
      translate([0, 0, -wall_thickness]){
        cylinder(d=10, h=base_h-mount_front_h, center=true);
      }
      rotate([0, 90, 0]){
        cylinder(d=3.2, h=40, center=true);
      }
    }
  }
  x1a=base_l/2-base_mount_edge-base_mount_l-mount_front_l;
  y1a=-mount_front_w/2;
  x2a=base_l/2-base_mount_edge-base_mount_l-mount_front_l-mount_front_w/2-2;
  y2a=-mount_front_w/2;
  x3a=base_l/2-base_mount_edge-base_mount_l-mount_front_l-mount_front_w/2-2;
  y3a=-mount_front_w/2-2;
  linear_extrude(height=base_h-mount_front_h){
    polygon(points=[[x1a,y1a],[x2a,y2a],[x3a,y3a]]);
  }
  linear_extrude(height=base_h-mount_front_h){
    polygon(points=[[x1a,-y1a],[x2a,-y2a],[x3a,-y3a]]);
  }
  

  x1=-base_l/2;
  y1=base_w/2;
  x2=base_l/2-base_mount_edge-base_mount_l-mount_front_l-mount_front_w/2;
  y2=(mount_front_w)/2;
  x3=base_l/2-base_mount_edge-base_mount_l-mount_front_l+mount_front_w/2;
  y3=(mount_front_w)/2;
  x4=-base_l/2;
  y4=base_w/2-15;
  
  linear_extrude(height=wall_thickness){
    polygon(points=[[x1,y1],[x2,y2],[x3,y3],[x4,y4]]);
  }
  linear_extrude(height=wall_thickness){
    polygon(points=[[x1,-y1],[x2,-y2],[x3,-y3],[x4,-y4]]);
  }
  for(i=[0:15:45]){
    translate([i, base_w/2+10, raspberry_mount_h/2]){
      difference(){
        cylinder(d=raspberry_mount_d, h=raspberry_mount_h, center=true);
        translate([0, 0, 2]){
          cylinder(d=raspberry_mount_screw_d, h=raspberry_mount_h, center=true);
        }
      }
    }
    translate([i, base_w/2+10, raspberry_mount_socket/2]){
     cylinder(d=raspberry_mount_d+2, h=raspberry_mount_socket, center=true);
    }
  }
  difference(){
    translate([0, 0, wall_thickness/2]){
      cube(size=[base_l, base_w, wall_thickness], center=true);
    }
    translate([base_l/2-cable_hole_l, cable_hole_w, 0]){
      cylinder(d=cable_hole_d, h=40, center=true);
    }
    translate([base_l/2-cable_hole_l, -cable_hole_w, 0]){
      cylinder(d=cable_hole_d, h=40, center=true);
    }
    
    translate([base_l/2 - base_mount_edge_2, 0, 0]){
      cylinder(d=3.2, h=40, center=true);
    }
    translate([base_l/2 - base_mount_edge_2, 11, 0]){
      cylinder(d=3.2, h=40, center=true);
    }
    translate([base_l/2 - base_mount_edge_2, -11, 0]){
      cylinder(d=3.2, h=40, center=true);
    }

    translate([body_mount_l/2, body_mount_w/2, 0]){
      cylinder(d=3.2, h=40, center=true);
    }
    translate([-body_mount_l/2, body_mount_w/2, 0]){
      cylinder(d=3.2, h=40, center=true);
    }
    translate([body_mount_l/2, -body_mount_w/2, 0]){
      cylinder(d=3.2, h=40, center=true);
    }
    translate([-body_mount_l/2, -body_mount_w/2, 0]){
      cylinder(d=3.2, h=40, center=true);
    }
    translate([base_l/2-raspberry_mount_edge, raspberry_mount_w/2, 0]){
      cylinder(d=raspberry_mount_d+raspberry_mount_d_tolerance, h=raspberry_mount_h, center=true);
    }
    translate([base_l/2-raspberry_mount_edge, -raspberry_mount_w/2, 0]){
      cylinder(d=raspberry_mount_d+raspberry_mount_d_tolerance, h=raspberry_mount_h, center=true);
    }
    translate([base_l/2-raspberry_mount_edge-raspberry_mount_l, raspberry_mount_w/2, 0]){
      cylinder(d=raspberry_mount_d+raspberry_mount_d_tolerance, h=raspberry_mount_h, center=true);
    }
    translate([base_l/2-raspberry_mount_edge-raspberry_mount_l, -raspberry_mount_w/2, 0]){
      cylinder(d=raspberry_mount_d+raspberry_mount_d_tolerance, h=raspberry_mount_h, center=true);
    }
  }
  translate([base_l/2-base_mount_edge, base_mount_w2/2, base_h/2]){
    base_mount();
  }
  translate([base_l/2-base_mount_edge, -base_mount_w2/2, base_h/2]){
    base_mount();
  }
  translate([base_l/2-base_mount_edge - base_mount_l, base_mount_w1/2, base_h/2]){
    base_mount();
  }
  translate([base_l/2-base_mount_edge - base_mount_l, -base_mount_w1/2, base_h/2]){
    base_mount();
  }
}

module base_mount(){
  enforcement_thickness = 2;
  enforcement_width = 5.5;
  difference(){
    cylinder(d=base_mount_d, h=base_h, center=true);
    translate([0, 0, base_h/2]){
      cylinder(d=base_mount_screw, h=base_h, center=true);
    }
  }
  for(alpha=[0:90:270]){
    echo(alpha);
    rotate([0, 0, alpha]){
      translate([base_mount_d/2 - 1, 0, 0]){
        linear_extrude(height=base_h, center=true, scale=[0, 1]){
          translate([enforcement_width/2, 0, 0]){
            square(size=[enforcement_width, enforcement_thickness], center=true);
          }
        }
      }
    }
  }
}