#! /usr/bin/perl
#Name: R7-client.perl
#Project: R7 control
#Homepage: www.HomoFaciens.de
#Author Norbert Heinz
#Version: 0.1
#Creation date: 23.08.2017
#Copyright: (c) 2011 by Norbert Heinz
#This program is free software you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation version 3 of the License.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program if not, see http://www.gnu.org/licenses/.
#
#Client script for R7-server.perl. Both scripts are used to operate a remote controlled rover and to do the picture upload from the webcam.

#echo 21= can also be echo 27= and vice versa (depending on Version of RaspberryPi) !!!!!!!!!!!!!!!!!!!!!!!



use strict;
use LWP::UserAgent;
use HTTP::Request::Common;
use Getopt::Long;
use File::Basename;
use LWP::Simple;
use Time::HiRes qw(usleep nanosleep);

$|=1 ;

my $pic_count = 0;
my $file_in = "002.jpg";
my $char_in = "";
my $i;
my @file_info;
my $file_size = 0;
my $char_temp = "";
my $wait_count = 0;
my $driveDistance = 0;
my $driveDirection = 1;
my $make_snapshot = 0;
my $last_watchdog_call = 0;
my ($seconds, $minutes, $hours, $day_of_month, $month, $year, $weekday, $day_of_year, $summertime) = localtime(time);
my $script_answer = "";
my $upload_command = "";
my $upload_status = 0;
my $lightStrength = -1;
my $IRID = "000";
my $IRCommand = 0;
my $stream = 0;
my $CommandStartTime = 0;
my $firstAccessTime = 0;
my $lastAccessTime = 0;
my $batteryStatus = 0;


#Password must be identically in SendCurrentCommand.pl and fileupload.pl stored at the Internet server.
my $password = "YOURPASSWORD";

my $url_file_upload = "http://192.168.2.17/perl/R7/fileupload.pl";
my $url_ask_command = 'http://192.168.2.17/perl/R7/SendCurrentCommand.pl?passwd=' . $password ;



&ExecuteCommandLine("killall -s SIGKILL R7C1", 0);
&ExecuteCommandLine("./R7C1 h 0", 0); #Light off

&ExecuteCommandLine("./stream-stop.sh", 0);


$script_answer = get $url_ask_command . "&status=FirstStart" . "&IRID=" . $IRID . "&STREAM=" . $stream . "&BATTERY=" . $batteryStatus;
sleep(2);


while (1 == 1){

  &WatchDogFile(0);

  $script_answer = get $url_ask_command . "&status=waiting" . "&IRID=" . $IRID . "&STREAM=" . $stream . "&BATTERY=" . $batteryStatus;
  $stream=0;
  print "2.) script_answer=>" . $script_answer . "<, url_ask_command=>" . $url_ask_command . "&status=waiting" . "&IRID=" . $IRID . "&STREAM=" . $stream . "&BATTERY=" . $batteryStatus . "<\n";

  while($script_answer eq "no job available"){
    #&ExecuteCommandLine("./R7C1 s 254", 0);#Tell Arduino that Raspberry is still running
    if($wait_count == 20){# Sleep
      &ExecuteCommandLine("./R7C1 h 0", 0);  #Turn light off
    }

    $wait_count ++;
    print "wait_count = $wait_count\n";
    sleep(2);

    if($firstAccessTime > 0 && (time() - $firstAccessTime > 300 || time() - $lastAccessTime > 40)){
      print "Timeout 2\n";
      $script_answer = get $url_ask_command . "&status=FinishedAll" . "&IRID=" . $IRID . "&STREAM=" . $stream . "&BATTERY=" . $batteryStatus;
      $firstAccessTime = 0;
      ExecuteCommandLine("./stream-stop.sh", 0);
      $stream = 0;
      sleep(2);
      &TransmitLastSnapshot();
    }
    else{
      $script_answer = get $url_ask_command . "&status=waiting" . "&IRID=" . $IRID . "&STREAM=" . $stream . "&BATTERY=" . $batteryStatus;
    }
    $stream=0;

    print "3.) script_answer=>" . $script_answer . "<\n";
    &WatchDogFile(0);
  }

  if($script_answer eq "reset jobs"){
    $lastAccessTime = time();
    print "Resetting job pipeline\n";
    $stream = 0;
    ExecuteCommandLine("./stream-stop.sh", 0);
  }

  if(substr($script_answer, 0, 5) eq "Comm="){
    $lastAccessTime = time();
    if($firstAccessTime == 0){
      $firstAccessTime = time();
      ExecuteCommandLine("./stream.sh", 1);
      sleep (5);#Give ffserver time to start
      $stream = 4;
    }
    $CommandStartTime = time();
    $wait_count = 0;
    print "\n\nReceived command '" . $script_answer . "'\n\n";


    if(substr($script_answer, 14, 1) eq "4"){#Restart video stream
      print "Restart picture stream.\n";
      ExecuteCommandLine("./stream.sh", 1);
      $stream = 4;
    }

#     &LogFile("script_answer=" . $script_answer);
    if(substr($script_answer, 14, 1) eq "1" || substr($script_answer, 14, 1) eq "2"){#make snapshot
#      &LogFile("make_snapshot=" . $make_snapshot);
      $make_snapshot = 1;
      print "Uploading snapshot...\n";
      #sleep(2);
      ExecuteCommandLine("./stream-stop.sh", 0);
      $stream = 0;
      ExecuteCommandLine("rm *.jpg", 0);
      sleep(3);
      if(substr($script_answer, 14, 1) eq "2"){#snapshot CentPrize
        #Raspberry camera auto mode
        ExecuteCommandLine("raspistill -hf -vf --width 320 --height 240 --quality 70 --timeout 2000 -sh 0 --awb auto --metering average --nopreview --output 001.jpg", 0);
        #Raspberry camera outdoor
        #ExecuteCommandLine("raspistill -hf --width 320 --height 240 --quality 70 --timeout 500 -sh 0 --awb auto --ISO 100 -ss 2000 --metering average --nopreview --output 001.jpg", 0);
        #Raspberry camera indoor
        #ExecuteCommandLine("raspistill -hf --width 320 --height 240 --quality 70 --timeout 500 -sh 0 --awb auto --ISO 800 -br 70 -co 70 -ss 20000 --metering average --nopreview --output 001.jpg", 0);
        #ExecuteCommandLine("ffmpeg -vcodec mjpeg -f video4linux2 -i /dev/video0 -frames:v 10 -s 320x240 -q:v 1 %03d.jpg", 0);
      }
      else{
        #Raspberry camera auto mode
        ExecuteCommandLine("raspistill -hf -vf --width 1920 --height 1440 --quality 15 --timeout 2000 -sh 0 --awb auto --metering average --nopreview --output 001.jpg", 0);
        #Raspberry camera outdoor
        #ExecuteCommandLine("raspistill -hf --width 1920 --height 1440 --quality 15 --timeout 500 -sh 0 --awb auto --ISO 100 -ss 2000 --metering average --nopreview --output 001.jpg", 0);
        #Raspberry camera indoor
        #ExecuteCommandLine("raspistill -hf --width 1920 --height 1440 --quality 15 --timeout 500 -sh 0 --awb auto --ISO 800 -br 70 -co 70 -ss 20000 --metering average --nopreview --output 001.jpg", 0);
        #ExecuteCommandLine("ffmpeg -vcodec mjpeg -f video4linux2 -i /dev/video0 -frames:v 10 -s 1920x1080 -q:v 1 %03d.jpg", 0);
      }


      $file_in = "001.jpg";

      if(-e $file_in){
        #Open image file and start transfer to server
        # put timeouts, proxy etc into the useragent if needed
        print $url_file_upload . "\n";
        if(substr($script_answer, 14, 1) eq "2"){#snapshot CentPrize
          $upload_command = "CentPrize";
        }
        else{
          $upload_command = "snapshot";
        }
        my $ua  = LWP::UserAgent->new();
        my $req = POST $url_file_upload, Content_Type => 'form-data', Content => [submit => 1, myfile => [ $file_in ], passwd => $password, command => $upload_command];
        my $response = $ua->request($req);

        if ($response->is_success()) {
          print "OK: ", $response->content;
        }
        else {
          print "ERROR: " . $response->as_string;
          &LogFile("File upload failed!");
        }
      }
      else{
        &LogFile("File>" . $file_in . "< for snapshot not found.");
      }

      #sleep(5);#Wait some seconds until stream is up
      print "Restart picture stream.\n";
      ExecuteCommandLine("./stream.sh", 1);
      $stream = 4;

    }
    else{
      $make_snapshot = 0;
    }

    $lightStrength = substr($script_answer, 13, 1);

      if(substr($script_answer, 13, 1) eq "0"){#Lights off
        &ExecuteCommandLine("./R7C1 h 0", 0);
      }
      if(substr($script_answer, 13, 1) eq "1"){#Light low
        &ExecuteCommandLine("./R7C1 h 1", 0);
      }
      if(substr($script_answer, 13, 1) eq "2"){
        &ExecuteCommandLine("./R7C1 h 2", 0);
      }
      if(substr($script_answer, 13, 1) eq "3"){
        &ExecuteCommandLine("./R7C1 h 3", 0);
      }
      if(substr($script_answer, 13, 1) eq "4"){#Light highest possible
        &ExecuteCommandLine("./R7C1 h 4", 0);
      }


    $driveDistance = substr($script_answer, 7, 2);
    if($driveDistance < 0 | $driveDistance > 10){
      print "driveDistance=$driveDistance out of range!\n";
      $driveDistance = 0;
    }

    if($driveDistance > 0){
      if(substr($script_answer, 6, 1) eq "0"){#Forward straight
        $driveDistance *= 300000;
        &ExecuteCommandLine("./R7C1 f " . $driveDistance, 0);
        $driveDirection = 1;
      }
      if(substr($script_answer, 6, 1) eq "1"){#Forward left
        $driveDistance *= 300000;
        &ExecuteCommandLine("./R7C1 fl " . $driveDistance, 0);
        $driveDistance = 0;
      }
      if(substr($script_answer, 6, 1) eq "2"){#Forward right
        $driveDistance *= 300000;
        &ExecuteCommandLine("./R7C1 fr " . $driveDistance, 0);
        $driveDistance = 0;
      }
      if(substr($script_answer, 6, 1) eq "3"){#Backward straight
        $driveDistance *= 300000;
        &ExecuteCommandLine("./R7C1 b " . $driveDistance, 0);
        $driveDirection = 2;
      }
      if(substr($script_answer, 6, 1) eq "5"){#Backward right
        $driveDistance *= 300000;
        &ExecuteCommandLine("./R7C1 br " . $driveDistance, 0);
        $driveDirection = 2;
      }
      if(substr($script_answer, 6, 1) eq "4"){#Backward left
        $driveDistance *= 300000;
        &ExecuteCommandLine("./R7C1 bl " . $driveDistance, 0);
        $driveDirection = 2;
      }
    }



    print "Start processing...\n";
    while(time() - $CommandStartTime < 2){#Next command after at least ...s
      $i++;
    }
  }#if(substr($script_answer, 0, 5) eq "Comm="){


  usleep(500000); #Do nothing for 0.5 seconds before proceeding the next command
  if($firstAccessTime > 0 && (time() - $firstAccessTime > 300 || time() - $lastAccessTime > 40)){
    print "Timeout 2: firstAccessTime = $firstAccessTime\n";
    $script_answer = get $url_ask_command . "&status=FinishedAll" . "&IRID=" . $IRID . "&STREAM=" . $stream . "&BATTERY=" . $batteryStatus;
    $firstAccessTime = 0;
    ExecuteCommandLine("./stream-stop.sh", 0);
    sleep(2);
    $stream = 0;
    &TransmitLastSnapshot();
  }
  else{
    $script_answer = get $url_ask_command . "&status=finished" . "&IRID=" . $IRID . "&STREAM=" . $stream . "&BATTERY=" . $batteryStatus;
  }
  $stream=0;

  #&ExecuteCommandLine("./R7C1 s 254", 0);#Tell Arduino that Raspberry is still running

}#while (1 == 1)


print $pic_count . " files sent\n";



#++++++++++++++++++++++++ LogFile ++++++++++++++++++++++++++++++++++++++++++++++++++++++
sub LogFile{

  @file_info=stat("R7-client.log");

  ($seconds, $minutes, $hours, $day_of_month, $month, $year, $weekday, $day_of_year, $summertime) = localtime(time());

  if ($hours < 10){
    $hours = "0" . $hours;
  }
  if ($minutes < 10){
    $minutes = "0" . $minutes;
  }
  if ($seconds < 10){
    $seconds = "0" . $seconds;
  }
  $month += 1;
  $year += 1900;

  if ($month < 10){
    $month = "0" . $month;
  }
  if ($day_of_month < 10){
    $day_of_month = "0" . $day_of_month;
  }

  $char_temp=substr($day_of_month . "." . $month . "." . $year . "-" . $hours . ":" . $minutes . ":" . $seconds . "                    ", 0, 20);

  if ($file_info[7] > 1000000){
    `mv R7-client.log R7-client01.log`
  }

  open(LOGFILE, ">> R7-client.log") || die "Can not open file 'R7-client.log'!\n";
    print LOGFILE $char_temp . $_[0] . "\n";
  close(LOGFILE);

}



#++++++++++++++++++++++ ExecuteCommandLine ++++++++++++++++++++++++++++++++++++++++++++++++++
sub ExecuteCommandLine{
  print $_[0] . "\n";
  if($_[1] == 0){
    `$_[0]`;
  }
  else{
#    `$_[0]`;
    system ($_[0] . "&");
  }
}

#++++++++++++++++++++++ WatchDogFile +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
sub WatchDogFile{#Print a timestamp into a file to show watchdog-client.perl that script is still alive
  if(time() - $last_watchdog_call > 10){
    open(WATCHDOGFILE, "> watchdog-client.tmp") || die "Can not open file 'watchdog-client.tmp'!\n";
      print WATCHDOGFILE time() . "\n";
    close(WATCHDOGFILE);
    if($_[0] == 0){
      `mv watchdog-client.tmp watchdog-client.txt`;
    }
    $last_watchdog_call = time();
  }
}

#++++++++++++++++++++++ TransmitLastSnapshot +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
sub TransmitLastSnapshot{
  ExecuteCommandLine("rm *.jpg", 0);
  #Raspberry camera auto mode
  ExecuteCommandLine("raspistill -hf -vf --width 320 --height 240 --quality 10 --timeout 2000 -sh 0 --awb auto --metering average --nopreview --output 001.jpg", 0);
  #Raspberry camera outdoor
  #ExecuteCommandLine("raspistill -hf --width 320 --height 240 --quality 10 --timeout 500 -sh 0 --awb auto --ISO 100 -ss 2000 --metering average --nopreview --output 001.jpg", 0);
  #Raspi camera indoor
  #ExecuteCommandLine("raspistill -hf --width 320 --height 240 --quality 10 --timeout 500 -sh 0 --awb auto --ISO 800 -br 70 -co 70 -ss 20000 --metering average --nopreview --output 001.jpg", 0);
  #ExecuteCommandLine("ffmpeg -vcodec mjpeg -f video4linux2 -i /dev/video0 -frames:v 10 -s 320x240 -q:v 1 %03d.jpg", 0);
  $file_in = "001.jpg";

   #Search for latest file
  for($i=1; $i<100; $i++){
    $char_temp = "0" . $i . ".jpg";
    if($i < 10){$char_temp = "0" . $char_temp;}
    if(-e $char_temp){
      $file_in = $char_temp;
    }
    else{
      $i=100000;
    }
  }

  if(-e $file_in){
    #Open image file and start transfer to server
    # put timeouts, proxy etc into the useragent if needed
    print $url_file_upload . "\n";
    $upload_command = "finishedShot";
    my $ua  = LWP::UserAgent->new();
    my $req = POST $url_file_upload, Content_Type => 'form-data', Content => [submit => 1, myfile => [ $file_in ], passwd => $password, command => $upload_command];
    my $response = $ua->request($req);

    if ($response->is_success()) {
      print "OK: ", $response->content;
    }
    else {
      print "ERROR: " . $response->as_string;
      &LogFile("File upload failed!");
    }
  }
  else{
    &LogFile("File>" . $file_in . "< for snapshot not found.");
  }
}
















